@@ -3,8 +3,8 @@
  *
  * Derived from Linux version by Richard Tobin.
  *
- * $Id: bsd.c,v 1.30 2004/02/18 06:34:52 jerdfelt Exp $
- * $Name: V0_1_10 $
+ * $Id: bsd.c,v 1.33 2006/03/04 01:16:10 jerdfelt Exp $
+ * $Name:  $
  *
  * This library is covered by the LGPL, read LICENSE for details.
  */
@@ -25,6 +25,10 @@
  * 	for both read and write.
  */
 
+#if defined(__FreeBSD__) && !defined(__FreeBSD_kernel__)
+#define __FreeBSD_kernel__ __FreeBSD__
+#endif
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
@@ -142,7 +146,7 @@ int usb_os_open(usb_dev_handle *dev)
     USB_ERROR(-ENOMEM);
   dev->impl_info = info;
 
-#if __FreeBSD__
+#ifdef __FreeBSD_kernel__
   snprintf(ctlpath, PATH_MAX, "%s", dev->device->filename);
 #else
   snprintf(ctlpath, PATH_MAX, "%s.00", dev->device->filename);
@@ -255,7 +259,7 @@ static int ensure_ep_open(usb_dev_handle *dev, int ep, int mode)
   ep = UE_GET_ADDR(ep);
 
   if (info->ep_fd[ep] < 0) {
-#if __FreeBSD__
+#ifdef __FreeBSD_kernel__
     snprintf(buf, sizeof(buf) - 1, "%s.%d", dev->device->filename, ep);
 #else
     snprintf(buf, sizeof(buf) - 1, "%s.%02d", dev->device->filename, ep);
@@ -286,7 +290,7 @@ int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
   fd = ensure_ep_open(dev, ep, O_WRONLY);
   if (fd < 0) {
     if (usb_debug >= 2) {
-#if __FreeBSD__
+#ifdef __FreeBSD_kernel__
       fprintf (stderr, "usb_bulk_write: got negative open file descriptor for endpoint %d\n", UE_GET_ADDR(ep));
 #else
       fprintf (stderr, "usb_bulk_write: got negative open file descriptor for endpoint %02d\n", UE_GET_ADDR(ep));
@@ -302,7 +306,7 @@ int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes, int size,
 
   ret = write(fd, bytes, size);
   if (ret < 0)
-#if __FreeBSD__
+#ifdef __FreeBSD_kernel__
     USB_ERROR_STR(-errno, "error writing to bulk endpoint %s.%d: %s",
                   dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
 #else
@@ -324,7 +328,7 @@ int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
   fd = ensure_ep_open(dev, ep, O_RDONLY);
   if (fd < 0) {
     if (usb_debug >= 2) {
-#if __FreeBSD__
+#ifdef __FreeBSD_kernel__
       fprintf (stderr, "usb_bulk_read: got negative open file descriptor for endpoint %d\n", UE_GET_ADDR(ep));
 #else
       fprintf (stderr, "usb_bulk_read: got negative open file descriptor for endpoint %02d\n", UE_GET_ADDR(ep));
@@ -343,7 +347,7 @@ int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes, int size,
 
   ret = read(fd, bytes, size);
   if (ret < 0)
-#if __FreeBSD__
+#ifdef __FreeBSD_kernel__
     USB_ERROR_STR(-errno, "error reading from bulk endpoint %s.%d: %s",
                   dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
 #else
@@ -365,7 +369,7 @@ int usb_interrupt_write(usb_dev_handle *dev, int ep, char *bytes, int size,
   fd = ensure_ep_open(dev, ep, O_WRONLY);
   if (fd < 0) {
     if (usb_debug >= 2) {
-#if __FreeBSD__
+#ifdef __FreeBSD_kernel__
       fprintf (stderr, "usb_interrupt_write: got negative open file descriptor for endpoint %d\n", UE_GET_ADDR(ep));
 #else
       fprintf (stderr, "usb_interrupt_write: got negative open file descriptor for endpoint %02d\n", UE_GET_ADDR(ep));
@@ -382,7 +386,7 @@ int usb_interrupt_write(usb_dev_handle *dev, int ep, char *bytes, int size,
   do {
     ret = write(fd, bytes+sent, size-sent);
     if (ret < 0)
-#if __FreeBSD__
+#ifdef __FreeBSD_kernel__
       USB_ERROR_STR(-errno, "error writing to interrupt endpoint %s.%d: %s",
                     dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
 #else
@@ -407,7 +411,7 @@ int usb_interrupt_read(usb_dev_handle *dev, int ep, char *bytes, int size,
   fd = ensure_ep_open(dev, ep, O_RDONLY);
   if (fd < 0) {
     if (usb_debug >= 2) {
-#if __FreeBSD__
+#ifdef __FreeBSD_kernel__
       fprintf (stderr, "usb_interrupt_read: got negative open file descriptor for endpoint %d\n", UE_GET_ADDR(ep));
 #else
       fprintf (stderr, "usb_interrupt_read: got negative open file descriptor for endpoint %02d\n", UE_GET_ADDR(ep));
@@ -427,7 +431,7 @@ int usb_interrupt_read(usb_dev_handle *dev, int ep, char *bytes, int size,
   do {
     ret = read(fd, bytes+retrieved, size-retrieved);
     if (ret < 0)
-#if __FreeBSD__
+#ifdef __FreeBSD_kernel__
       USB_ERROR_STR(-errno, "error reading from interrupt endpoint %s.%d: %s",
                     dev->device->filename, UE_GET_ADDR(ep), strerror(errno));
 #else
@@ -457,7 +461,7 @@ int usb_control_msg(usb_dev_handle *dev, int requesttype, int request,
   USETW(req.ucr_request.wLength, size);
 
   req.ucr_data = bytes;
-  req.ucr_flags = 0;
+  req.ucr_flags = USBD_SHORT_XFER_OK;
 
   ret = ioctl(dev->fd, USB_SET_TIMEOUT, &timeout);
 #if (__NetBSD__ || __OpenBSD__)
@@ -531,6 +535,7 @@ int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
   for (device = 1; device < USB_MAX_DEVICES; device++) {
     struct usb_device_info di;
     struct usb_device *dev;
+    unsigned char device_desc[DEVICE_DESC_LENGTH];
     char buf[20];
 
     di.udi_addr = device;
@@ -543,7 +548,7 @@ int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
       /* best not to play with things we don't understand */
       continue;
 
-#if __FreeBSD__
+#ifdef __FreeBSD_kernel__
     snprintf(buf, sizeof(buf) - 1, "/dev/%s", di.udi_devnames[0]);
 #else
     snprintf(buf, sizeof(buf) - 1, "/dev/%s.00", di.udi_devnames[0]);
@@ -576,16 +581,13 @@ int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
     strncpy(dev->filename, buf, sizeof(dev->filename) - 1);
     dev->filename[sizeof(dev->filename) - 1] = 0;
 
-    if (ioctl(dfd, USB_GET_DEVICE_DESC, &dev->descriptor) < 0)
+    if (ioctl(dfd, USB_GET_DEVICE_DESC, device_desc) < 0)
       USB_ERROR_STR(-errno, "couldn't get device descriptor for %s: %s",
                     buf, strerror(errno));
 
     close(dfd);
 
-    USB_LE16_TO_CPU(dev->descriptor.bcdUSB);
-    USB_LE16_TO_CPU(dev->descriptor.idVendor);
-    USB_LE16_TO_CPU(dev->descriptor.idProduct);
-    USB_LE16_TO_CPU(dev->descriptor.bcdDevice);
+    usb_parse_descriptor(device_desc, "bbwbbbbwwwbbbb", &dev->descriptor);
 
     LIST_ADD(fdev, dev);
 
