@@ -65,23 +65,30 @@
 
 /* some defines */
 /* IOUSBInterfaceInferface */
-#if defined (kIOUSBInterfaceInterfaceID197)
+#if defined (kIOUSBInterfaceInterfaceID220)
 
-#warning "libusb being compiled for 10.3 or later"
+// #warning "libusb being compiled for 10.4 or later"
+#define usb_interface_t IOUSBInterfaceInterface220
+#define InterfaceInterfaceID kIOUSBInterfaceInterfaceID220
+#define InterfaceVersion 220
+
+#elif defined (kIOUSBInterfaceInterfaceID197)
+
+// #warning "libusb being compiled for 10.3 or later"
 #define usb_interface_t IOUSBInterfaceInterface197
 #define InterfaceInterfaceID kIOUSBInterfaceInterfaceID197
 #define InterfaceVersion 197
 
 #elif defined (kIOUSBInterfaceInterfaceID190)
 
-#warning "libusb being compiled for 10.2 or later"
+// #warning "libusb being compiled for 10.2 or later"
 #define usb_interface_t IOUSBInterfaceInterface190
 #define InterfaceInterfaceID kIOUSBInterfaceInterfaceID190
 #define InterfaceVersion 190
 
 #elif defined (kIOUSBInterfaceInterfaceID182)
 
-#warning "libusb being compiled for 10.1 or later"
+// #warning "libusb being compiled for 10.1 or later"
 #define usb_interface_t IOUSBInterfaceInterface182
 #define InterfaceInterfaceID kIOUSBInterfaceInterfaceID182
 #define InterfaceVersion 182
@@ -89,7 +96,7 @@
 #else
 
 /* No timeout functions available! Time to upgrade your os. */
-#warning "libusb being compiled without support for timout bulk functions! 10.0 and up"
+#warning "libusb being compiled without support for timeout bulk functions! 10.0 and up"
 #define usb_interface_t IOUSBInterfaceInterface
 #define InterfaceInterfaceID kIOUSBInterfaceInterfaceID
 #define LIBUSB_NO_TIMEOUT_INTERFACE
@@ -180,6 +187,8 @@ static char *darwin_error_str (int result) {
     return "transaction timed out";
   case kIOReturnBadArgument:
     return "invalid argument";
+  case kIOReturnAborted:
+    return "transaction aborted";
   default:
     return "unknown error";
   }
@@ -399,6 +408,7 @@ static int get_endpoints (struct darwin_dev_handle *device)
     USB_ERROR_STR ( -ret, "get_endpoints: can't get number of endpoints for interface" );
   }
 
+  free (device->endpoint_addrs);
   device->endpoint_addrs = calloc (sizeof (unsigned char), numep);
 
   /* iterate through pipe references */
@@ -718,6 +728,12 @@ int usb_set_altinterface(usb_dev_handle *dev, int alternate)
 
   dev->altsetting = alternate;
 
+  result = get_endpoints (device);
+  if (result) {
+    /* this should not happen */
+    USB_ERROR_STR ( result, "usb_set_altinterface: could not build endpoint table");
+  }
+
   return 0;
 }
 
@@ -752,9 +768,9 @@ static void rw_completed(void *refcon, io_return_t result, void *io_size)
   struct rw_complete_arg *rw_arg = (struct rw_complete_arg *)refcon;
 
   if (usb_debug > 2)
-    fprintf(stderr, "io async operation completed: %s, size = %lu\n", darwin_error_str(result),
-	    (UInt32)io_size);
-  
+    fprintf(stderr, "io async operation completed: %s, size=%lu, result=0x%08x\n", darwin_error_str(result),
+	    (UInt32)io_size, result);
+
   rw_arg->io_size = (UInt32)io_size;
   rw_arg->result  = result;
 
@@ -824,6 +840,7 @@ static int usb_bulk_transfer (usb_dev_handle *dev, int ep, char *bytes, int size
     /* wait for write to complete */
     if (CFRunLoopRunInMode(kCFRunLoopDefaultMode, (timeout+999)/1000, true) == kCFRunLoopRunTimedOut) {
       (*(device->interface))->AbortPipe(device->interface, pipeRef);
+      CFRunLoopRunInMode(kCFRunLoopDefaultMode, 0, true); /* Pick up aborted callback */
       if (usb_debug)
 	fprintf(stderr, "usb_bulk_read: input timed out\n");
     }
@@ -832,7 +849,8 @@ static int usb_bulk_transfer (usb_dev_handle *dev, int ep, char *bytes, int size
   CFRunLoopRemoveSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
   
   /* Check the return code of both the write and completion functions. */
-  if (result != kIOReturnSuccess || rw_arg.result != kIOReturnSuccess) {
+  if (result != kIOReturnSuccess || (rw_arg.result != kIOReturnSuccess && 
+      rw_arg.result != kIOReturnAborted) ) {
     int error_code;
     char *error_str;
 
@@ -1041,6 +1059,8 @@ int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
 
 
   while ((device = usb_get_next_device (deviceIterator, &location)) != NULL) {
+    unsigned char device_desc[DEVICE_DESC_LENGTH];
+
     result = (*(device))->GetDeviceAddress(device, (USBDeviceAddress *)&address);
 
     if (usb_debug >= 2)
@@ -1056,13 +1076,10 @@ int usb_os_find_devices(struct usb_bus *bus, struct usb_device **devices)
 
       dev->bus = bus;
 
-      req.pData = &(dev->descriptor);
+      req.pData = device_desc;
       result = (*(device))->DeviceRequest(device, &req);
 
-      USB_LE16_TO_CPU(dev->descriptor.bcdUSB);
-      USB_LE16_TO_CPU(dev->descriptor.idVendor);
-      USB_LE16_TO_CPU(dev->descriptor.idProduct);
-      USB_LE16_TO_CPU(dev->descriptor.bcdDevice);
+      usb_parse_descriptor(device_desc, "bbwbbbbwwwbbbb", &dev->descriptor);
 
       sprintf(dev->filename, "%03i-%04x-%04x-%02x-%02x", address,
 	      dev->descriptor.idVendor, dev->descriptor.idProduct,
